@@ -29,13 +29,13 @@ extern "C" {
  * @{
  */
 
-/** Add a new function to a server instance.
+/** 
+  Add a new function to a server instance.
  */
 GEARMAN_API
-gearman_server_function_st *
-gearman_server_function_get(gearman_server_st *server,
-                            const char *function_name,
-                            size_t function_name_size);
+  gearman_server_function_st * gearman_server_function_get(gearman_server_st *server,
+                                                           const char *function_name,
+                                                           size_t function_name_size);
 
 /**
  * Free a server function structure.
