@@ -11,11 +11,12 @@
  * @brief Gearmand Thread Definitions
  */
 
+#include <config.h>
 #include <libgearman-server/common.h>
 #include <libgearman-server/gearmand.h>
 
-#include <assert.h>
-#include <errno.h>
+#include <cassert>
+#include <cerrno>
 
 #include <libgearman-server/list.h>
 
