@@ -60,17 +60,6 @@
  * @{
  */
 
-/**
- * Generate hash key for job handles and unique IDs.
- */
-static uint32_t _server_job_hash(const char *key, size_t key_size);
-
-/**
- * Appends a worker onto the end of a list of workers.
- */
-static inline void _server_con_worker_list_append(gearman_server_worker_st *list,
-                                                  gearman_server_worker_st *worker);
-
 /**
  * Get a server job structure from the unique ID. If data_size is non-zero,
  * then unique points to the workload data and not a real unique key.
@@ -157,8 +146,11 @@ gearman_server_job_add_reducer(gearman_server_st *server,
 
   if (server_job == NULL)
   {
-    if (server_function->max_queue_size > 0 &&
-        server_function->job_total >= server_function->max_queue_size)
+    gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "Comparing queue %u to limit %u for priority %u",
+      server_function->job_total, server_function->max_queue_size[priority],
+      priority);
+    if (server_function->max_queue_size[priority] > 0 &&
+        server_function->job_total >= server_function->max_queue_size[priority])
     {
       *ret_ptr= GEARMAN_JOB_QUEUE_FULL;
       return NULL;
@@ -190,7 +182,7 @@ gearman_server_job_add_reducer(gearman_server_st *server,
                              (int)unique_size, unique);
     if (checked_length >= GEARMAN_UNIQUE_SIZE || checked_length < 0)
     {
-      gearmand_log_error(GEARMAN_DEFAULT_LOG_PARAM, "We recieved a unique beyond GEARMAN_UNIQUE_SIZE: %.*s", (int)unique_size, unique);
+      gearmand_log_error(GEARMAN_DEFAULT_LOG_PARAM, "We received a unique beyond GEARMAN_UNIQUE_SIZE: %.*s", (int)unique_size, unique);
     }
 
     server->job_handle_count++;
@@ -370,177 +362,6 @@ void gearman_server_job_free(gearman_server_job_st *server_job)
   }
 }
 
-gearman_server_job_st *gearman_server_job_get(gearman_server_st *server,
-                                              const char *job_handle,
-                                              gearman_server_con_st *worker_con)
-{
-  uint32_t key;
-
-  key= _server_job_hash(job_handle, strlen(job_handle));
-
-  for (gearman_server_job_st *server_job= server->job_hash[key % GEARMAND_JOB_HASH_SIZE];
-       server_job != NULL; server_job= server_job->next)
-  {
-    if (server_job->job_handle_key == key &&
-        !strcmp(server_job->job_handle, job_handle))
-    {
-      /* Check to make sure the worker asking for the job still owns the job. */
-      if (worker_con != NULL &&
-          (server_job->worker == NULL || server_job->worker->con != worker_con))
-      {
-        return NULL;
-      }
-
-      return server_job;
-    }
-  }
-
-  return NULL;
-}
-
-gearman_server_job_st *
-gearman_server_job_peek(gearman_server_con_st *server_con)
-{
-  for (gearman_server_worker_st *server_worker= server_con->worker_list;
-       server_worker != NULL;
-       server_worker= server_worker->con_next)
-  {
-    if (server_worker->function->job_count != 0)
-    {
-      for (gearmand_job_priority_t priority= GEARMAND_JOB_PRIORITY_HIGH;
-           priority != GEARMAND_JOB_PRIORITY_MAX; priority++)
-      {
-        gearman_server_job_st *server_job;
-        server_job= server_worker->function->job_list[priority];
-
-        int64_t current_time= (int64_t)time(NULL);
-
-        while(server_job && 
-              server_job->when != 0 && 
-              server_job->when > current_time)
-        {
-          server_job= server_job->function_next;  
-        }
-        
-        if (server_job != NULL)
-        {
-
-          if (server_job->ignore_job)
-          {
-            /* This is only happens when a client disconnects from a foreground
-              job. We do this because we don't want to run the job anymore. */
-            server_job->ignore_job= false;
-
-            gearman_server_job_free(gearman_server_job_take(server_con));
-
-            return gearman_server_job_peek(server_con);
-          }
-        
-          return server_job;
-        }
-      }
-    }
-  }
-  
-
-  return NULL;
-}
-
-static inline void _server_con_worker_list_append(gearman_server_worker_st *list,
-                                                  gearman_server_worker_st *worker)
-{
-  worker->con_prev= NULL;
-  worker->con_next= list;
-  while (worker->con_next != NULL)
-  {
-    worker->con_prev= worker->con_next;
-    worker->con_next= worker->con_next->con_next;
-  }
-  if (worker->con_prev)
-    worker->con_prev->con_next= worker;
-}
-
-gearman_server_job_st *gearman_server_job_take(gearman_server_con_st *server_con)
-{
-  for (gearman_server_worker_st *server_worker= server_con->worker_list; server_worker; server_worker= server_worker->con_next)
-  {
-    gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "Jobs available %lu", (unsigned long)(server_worker->function->job_count));
-    if (server_worker->function->job_count)
-    {
-      if (server_worker == NULL)
-      {
-        return NULL;
-      }
-
-      if (Server->flags.round_robin)
-      {
-        GEARMAN_LIST_DEL(server_con->worker, server_worker, con_)
-        _server_con_worker_list_append(server_con->worker_list, server_worker);
-        ++server_con->worker_count;
-        if (server_con->worker_list == NULL)
-        {
-          server_con->worker_list= server_worker;
-        }
-      }
-
-      gearmand_job_priority_t priority;
-      for (priority= GEARMAND_JOB_PRIORITY_HIGH; priority < GEARMAND_JOB_PRIORITY_LOW; priority++)
-      {
-        if (server_worker->function->job_list[priority])
-        {
-          break;
-        }
-      }
-
-      gearman_server_job_st *server_job= server_job= server_worker->function->job_list[priority];
-      gearman_server_job_st *previous_job= server_job;
-  
-      int64_t current_time= (int64_t)time(NULL);
-  
-      while (server_job && server_job->when != 0 && server_job->when > current_time)
-      {
-        previous_job= server_job;
-        server_job= server_job->function_next;  
-      }
-  
-      if (server_job)
-      { 
-        if (server_job->function->job_list[priority] == server_job)
-        {
-          // If it's the head of the list, advance it
-          server_job->function->job_list[priority]= server_job->function_next;
-        }
-        else
-        {
-          // Otherwise, just remove the item from the list
-          previous_job->function_next= server_job->function_next;
-        }
-        
-        // If it's the tail of the list, move the tail back
-        if (server_job->function->job_end[priority] == server_job)
-        {
-          server_job->function->job_end[priority]= previous_job;
-        }
-        server_job->function->job_count--;
-
-        server_job->worker= server_worker;
-        GEARMAN_LIST_ADD(server_worker->job, server_job, worker_)
-        server_job->function->job_running++;
-
-        if (server_job->ignore_job)
-        {
-          gearman_server_job_free(server_job);
-          return gearman_server_job_take(server_con);
-        }
-        
-        return server_job;
-      }
-    }
-  }
-  
-  return NULL;
-}
-
 gearmand_error_t gearman_server_job_queue(gearman_server_job_st *job)
 {
   gearman_server_client_st *client;
@@ -648,25 +469,6 @@ gearmand_error_t gearman_server_job_queue(gearman_server_job_st *job)
  * Private definitions
  */
 
-static uint32_t _server_job_hash(const char *key, size_t key_size)
-{
-  const char *ptr= key;
-  int32_t value= 0;
-
-  while (key_size--)
-  {
-    value += (int32_t)*ptr++;
-    value += (value << 10);
-    value ^= (value >> 6);
-  }
-
-  value += (value << 3);
-  value ^= (value >> 11);
-  value += (value << 15);
-
-  return (uint32_t)(value == 0 ? 1 : value);
-}
-
 static gearman_server_job_st *
 _server_job_get_unique(gearman_server_st *server, uint32_t unique_key,
                        gearman_server_function_st *server_function,
