@@ -24,12 +24,6 @@
  */
 
 
-#define TEXT_SUCCESS "OK\r\n"
-#define TEXT_ERROR_ARGS "ERR INVALID_ARGUMENTS An+incomplete+set+of+arguments+was+sent+to+this+command+%.*s\r\n"
-#define TEXT_ERROR_CREATE_FUNCTION "ERR CREATE_FUNCTION %.*s\r\n"
-#define TEXT_ERROR_UNKNOWN_COMMAND "ERR UNKNOWN_COMMAND Unknown+server+command%.*s\r\n"
-#define TEXT_ERROR_INTERNAL_ERROR "ERR UNKNOWN_ERROR\r\n"
-
 /**
  * @addtogroup gearman_server_private Private Server Functions
  * @ingroup gearman_server
@@ -53,12 +47,6 @@ static gearmand_error_t _server_error_packet(gearman_server_con_st *server_con,
                                              const char *error_code,
                                              const char *error_string);
 
-/**
- * Process text commands for a connection.
- */
-static gearmand_error_t _server_run_text(gearman_server_con_st *server_con,
-                                         gearmand_packet_st *packet);
-
 /**
  * Send work result packets with data back to clients.
  */
@@ -158,7 +146,7 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
       {
         gearman_server_client_free(server_client);
         gearmand_gerror("gearman_server_job_add", ret);
-        return ret;
+        return _server_error_packet(server_con, "queue_insert_error", gearmand_strerror(ret));
       }
 
       /* Queue the job created packet. */
@@ -267,7 +255,7 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
     {
       gearman_server_client_free(server_client);
       gearmand_gerror("gearman_server_job_add", ret);
-      return ret;
+      return _server_error_packet(server_con, "queue_insert_error", gearmand_strerror(ret));
     }
 
     /* Queue the job created packet. */
@@ -400,8 +388,7 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
   case GEARMAN_COMMAND_CAN_DO_TIMEOUT:
     if (gearman_server_worker_add(server_con, (char *)(packet->arg[0]),
                                   packet->arg_size[0] - 1,
-                                  (in_port_t)atoi((char *)(packet->arg[1])))
-         == NULL)
+                                  0) == NULL)
     {
       return GEARMAN_MEMORY_ALLOCATION_FAILURE;
     }
@@ -422,6 +409,8 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
     if (server_job == NULL)
     {
       server_con->is_sleeping= true;
+      /* Remove any timeouts while sleeping */
+      gearman_server_con_delete_timeout(server_con);
     }
     else
     {
@@ -519,11 +508,20 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
       gearmand_gerror("gearman_server_io_packet_add", ret);
 
       if (server_job)
+      {
         return gearman_server_job_queue(server_job);
+      }
 
       return ret;
     }
 
+    /* Since job is assigned, we should respect function timeout */
+    if (server_job != NULL)
+    {
+      gearman_server_con_add_job_timeout(server_con, server_job);
+    }
+
+
     break;
 
   case GEARMAN_COMMAND_WORK_DATA:
@@ -710,7 +708,7 @@ gearmand_error_t gearman_server_run_command(gearman_server_con_st *server_con,
     break;
 
   case GEARMAN_COMMAND_TEXT:
-    return _server_run_text(server_con, packet);
+    return server_run_text(server_con, packet);
 
   case GEARMAN_COMMAND_UNUSED:
   case GEARMAN_COMMAND_NOOP:
@@ -815,365 +813,6 @@ static gearmand_error_t _server_error_packet(gearman_server_con_st *server_con,
                                       (size_t)strlen(error_string), NULL);
 }
 
-static gearmand_error_t _server_run_text(gearman_server_con_st *server_con,
-                                         gearmand_packet_st *packet)
-{
-  char *data;
-  char *new_data;
-  size_t size;
-  size_t total;
-  int max_queue_size;
-  gearman_server_thread_st *thread;
-  gearman_server_con_st *con;
-  gearman_server_worker_st *worker;
-  gearman_server_packet_st *server_packet;
-  int checked_length;
-
-  data= (char *)(char *)malloc(GEARMAN_TEXT_RESPONSE_SIZE);
-  if (data == NULL)
-  {
-    gearmand_perror("malloc");
-    return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-  }
-  total= GEARMAN_TEXT_RESPONSE_SIZE;
-  data[0]= 0;
-
-  if (packet->argc)
-  {
-    gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "text command %.*s", packet->arg_size[0],  packet->arg[0]);
-  }
-
-  if (packet->argc == 0)
-  {
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_UNKNOWN_COMMAND, 4, "NULL");
-  }
-  else if (!strcasecmp("workers", (char *)(packet->arg[0])))
-  {
-    size= 0;
-
-    for (thread= Server->thread_list; thread != NULL;
-         thread= thread->next)
-    {
-      int error;
-      if (! (error= pthread_mutex_lock(&thread->lock)))
-      {
-        for (con= thread->con_list; con != NULL; con= con->next)
-        {
-          if (con->_host == NULL)
-            continue;
-
-          if (size > total)
-            size= total;
-
-          /* Make sure we have at least GEARMAN_TEXT_RESPONSE_SIZE bytes. */
-          if (size + GEARMAN_TEXT_RESPONSE_SIZE > total)
-          {
-            new_data= (char *)realloc(data, total + GEARMAN_TEXT_RESPONSE_SIZE);
-            if (new_data == NULL)
-            {
-              (void) pthread_mutex_unlock(&thread->lock);
-              gearmand_perror("realloc");
-              gearmand_debug("free");
-              free(data);
-              return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-            }
-
-            data= new_data;
-            total+= GEARMAN_TEXT_RESPONSE_SIZE;
-          }
-
-          checked_length= snprintf(data + size, total - size, "%d %s %s :",
-                                   con->con.fd, con->_host, con->id);
-
-          if ((size_t)checked_length > total - size || checked_length < 0)
-          {
-            (void) pthread_mutex_unlock(&thread->lock);
-            gearmand_debug("free");
-            free(data);
-            gearmand_perror("snprintf");
-            return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-          }
-
-          size+= (size_t)checked_length;
-          if (size > total)
-            continue;
-
-          for (worker= con->worker_list; worker != NULL; worker= worker->con_next)
-          {
-            checked_length= snprintf(data + size, total - size, " %.*s",
-                                     (int)(worker->function->function_name_size),
-                                     worker->function->function_name);
-
-            if ((size_t)checked_length > total - size || checked_length < 0)
-            {
-              (void) pthread_mutex_unlock(&thread->lock);
-              gearmand_debug("free");
-              free(data);
-              gearmand_perror("snprintf");
-              return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-            }
-
-            size+= (size_t)checked_length;
-            if (size > total)
-              break;
-          }
-
-          if (size > total)
-            continue;
-
-          checked_length= snprintf(data + size, total - size, "\n");
-          if ((size_t)checked_length > total - size || checked_length < 0)
-          {
-            (void) pthread_mutex_unlock(&thread->lock);
-            gearmand_debug("free");
-            free(data);
-            gearmand_perror("snprintf");
-            return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-          }
-          size+= (size_t)checked_length;
-        }
-
-        (void) pthread_mutex_unlock(&thread->lock);
-      }
-      else
-      {
-        errno= error;
-        gearmand_error("pthread_mutex_lock");
-        assert(! "pthread_mutex_lock");
-      }
-    }
-
-    if (size < total)
-    {
-      checked_length= snprintf(data + size, total - size, ".\n");
-      if ((size_t)checked_length > total - size || checked_length < 0)
-      {
-        gearmand_perror("snprintf");
-        return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-      }
-    }
-  }
-  else if (!strcasecmp("status", (char *)(packet->arg[0])))
-  {
-    size= 0;
-
-    gearman_server_function_st *function;
-    for (function= Server->function_list; function != NULL;
-         function= function->next)
-    {
-      if (size + GEARMAN_TEXT_RESPONSE_SIZE > total)
-      {
-        new_data= (char *)realloc(data, total + GEARMAN_TEXT_RESPONSE_SIZE);
-        if (new_data == NULL)
-        {
-          gearmand_perror("realloc");
-          gearmand_debug("free");
-          free(data);
-          return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-        }
-
-        data= new_data;
-        total+= GEARMAN_TEXT_RESPONSE_SIZE;
-      }
-
-      checked_length= snprintf(data + size, total - size, "%.*s\t%u\t%u\t%u\n",
-                               (int)(function->function_name_size),
-                               function->function_name, function->job_total,
-                               function->job_running, function->worker_count);
-
-      if ((size_t)checked_length > total - size || checked_length < 0)
-      {
-        gearmand_perror("snprintf");
-        gearmand_debug("free");
-        free(data);
-        return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-      }
-
-      size+= (size_t)checked_length;
-      if (size > total)
-        size= total;
-    }
-
-    if (size < total)
-    {
-      checked_length= snprintf(data + size, total - size, ".\n");
-      if ((size_t)checked_length > total - size || checked_length < 0)
-      {
-        gearmand_perror("snprintf");
-        gearmand_debug("free");
-        free(data);
-        return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-      }
-    }
-  }
-  else if (!strcasecmp("create", (char *)(packet->arg[0])))
-  {
-    if (packet->argc == 3 && !strcasecmp("function", (char *)(packet->arg[1])))
-    {
-      gearman_server_function_st *function;
-      function= gearman_server_function_get(Server, (char *)(packet->arg[2]), packet->arg_size[2] -2);
-
-      if (function)
-      {
-        snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_SUCCESS);
-      }
-      else
-      {
-        snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_CREATE_FUNCTION,
-                 (int)packet->arg_size[2], (char *)(packet->arg[2]));
-      }
-    }
-    else
-    {
-      // create
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_ARGS, (int)packet->arg_size[0], (char *)(packet->arg[0]));
-    }
-  }
-  else if (! strcasecmp("drop", (char *)(packet->arg[0])))
-  {
-    if (packet->argc == 3 && !strcasecmp("function", (char *)(packet->arg[1])))
-    {
-      bool success= false;
-      for (gearman_server_function_st *function= Server->function_list; function != NULL; function= function->next)
-      {
-        if (! strcasecmp(function->function_name, (char *)(packet->arg[2])))
-        {
-          success++;
-          if (function->worker_count == 0 && function->job_running == 0)
-          {
-            gearman_server_function_free(Server, function);
-            snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_SUCCESS);
-          }
-          else
-          {
-            snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, "ERR there are still connected workers or executing clients\r\n");
-          }
-          break;
-        }
-      }
-
-      if (! success)
-      {
-        snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, "ERR function not found\r\n");
-        gearmand_debug(data);
-      }
-    }
-    else
-    {
-      // drop
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_ARGS, (int)packet->arg_size[0], (char *)(packet->arg[0]));
-    }
-  }
-  else if (!strcasecmp("maxqueue", (char *)(packet->arg[0])))
-  {
-    if (packet->argc == 1)
-    {
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_ARGS, (int)packet->arg_size[0], (char *)(packet->arg[0]));
-    }
-    else
-    {
-      if (packet->argc == 2)
-        max_queue_size= GEARMAN_DEFAULT_MAX_QUEUE_SIZE;
-      else
-      {
-        max_queue_size= atoi((char *)(packet->arg[2]));
-        if (max_queue_size < 0)
-          max_queue_size= 0;
-      }
-
-      gearman_server_function_st *function;
-      for (function= Server->function_list;
-           function != NULL; function= function->next)
-      {
-        if (strlen((char *)(packet->arg[1])) == function->function_name_size &&
-            !memcmp(packet->arg[1], function->function_name,
-                    function->function_name_size))
-        {
-          function->max_queue_size= (uint32_t)max_queue_size;
-        }
-      }
-
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_SUCCESS);
-    }
-  }
-  else if (strcasecmp("getpid", (char *)(packet->arg[0])) == 0)
-  {
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, "OK %d\n", (int)getpid());
-  }
-  else if (!strcasecmp("shutdown", (char *)(packet->arg[0])))
-  {
-    if (packet->argc == 1)
-    {
-      Server->shutdown= true;
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_SUCCESS);
-    }
-    else if (packet->argc == 2 &&
-             !strcasecmp("graceful", (char *)(packet->arg[1])))
-    {
-      Server->shutdown_graceful= true;
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_SUCCESS);
-    }
-    else
-    {
-      // shutdown
-      snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_ARGS, (int)packet->arg_size[0], (char *)(packet->arg[0]));
-    }
-  }
-  else if (!strcasecmp("verbose", (char *)(packet->arg[0])))
-  {
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, "OK %s\n", gearmand_verbose_name(Gearmand()->verbose));
-  }
-  else if (!strcasecmp("version", (char *)(packet->arg[0])))
-  {
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, "OK %s\n", PACKAGE_VERSION);
-  }
-  else
-  {
-    gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "Failed to find command %.*s(%llu)", packet->arg_size[0], packet->arg[0], 
-                       (unsigned long long)packet->arg_size[0]);
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_UNKNOWN_COMMAND, (int)packet->arg_size[0], (char *)(packet->arg[0]));
-  }
-
-  server_packet= gearman_server_packet_create(server_con->thread, false);
-  if (server_packet == NULL)
-  {
-    gearmand_debug("free");
-    free(data);
-    return GEARMAN_MEMORY_ALLOCATION_FAILURE;
-  }
-
-  gearmand_packet_init(&(server_packet->packet), GEARMAN_MAGIC_TEXT, GEARMAN_COMMAND_TEXT);
-
-  server_packet->packet.magic= GEARMAN_MAGIC_TEXT;
-  server_packet->packet.command= GEARMAN_COMMAND_TEXT;
-  server_packet->packet.options.complete= true;
-  server_packet->packet.options.free_data= true;
-
-  if (data[0] == 0)
-  {
-    snprintf(data, GEARMAN_TEXT_RESPONSE_SIZE, TEXT_ERROR_INTERNAL_ERROR);
-  }
-  server_packet->packet.data= data;
-  server_packet->packet.data_size= strlen(data);
-
-  int error;
-  if (! (error= pthread_mutex_lock(&server_con->thread->lock)))
-  {
-    GEARMAN_FIFO_ADD(server_con->io_packet, server_packet,);
-    (void) pthread_mutex_unlock(&server_con->thread->lock);
-  }
-  else
-  {
-    errno= error;
-    gearmand_perror("pthread_mutex_lock");
-    assert(!"pthread_mutex_lock");
-  }
-
-  gearman_server_con_io_add(server_con);
-
-  return GEARMAN_SUCCESS;
-}
-
 static gearmand_error_t
 _server_queue_work_data(gearman_server_job_st *server_job,
                         gearmand_packet_st *packet, gearman_command_t command)
