@@ -1,22 +1,37 @@
 /*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
- * 
- *  libtest
  *
- *  Copyright (C) 2011 Data Differential, http://datadifferential.com/
+ *  Data Differential YATL (i.e. libtest)  library
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 3 of the License, or (at your option) any later version.
+ *  Copyright (C) 2012 Data Differential, http://datadifferential.com/
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -76,13 +91,14 @@ Server* server_startup_st::pop_server()
   return tmp;
 }
 
-bool server_startup_st::shutdown(uint32_t number_of_host)
+// host_to_shutdown => host number to shutdown in array
+bool server_startup_st::shutdown(uint32_t host_to_shutdown)
 {
-  if (servers.size() > number_of_host)
+  if (servers.size() > host_to_shutdown)
   {
-    Server* tmp= servers[number_of_host];
+    Server* tmp= servers[host_to_shutdown];
 
-    if (tmp and tmp->has_pid() and not tmp->kill(tmp->pid()))
+    if (tmp and tmp->kill() == false)
     { }
     else
     {
@@ -93,7 +109,7 @@ bool server_startup_st::shutdown(uint32_t number_of_host)
   return false;
 }
 
-void server_startup_st::shutdown_and_remove()
+void server_startup_st::clear()
 {
   for (std::vector<Server *>::iterator iter= servers.begin(); iter != servers.end(); iter++)
   {
@@ -102,15 +118,33 @@ void server_startup_st::shutdown_and_remove()
   servers.clear();
 }
 
-void server_startup_st::shutdown()
+bool server_startup_st::check() const
 {
-  for (std::vector<Server *>::iterator iter= servers.begin(); iter != servers.end(); iter++)
+  bool success= true;
+  for (std::vector<Server *>::const_iterator iter= servers.begin(); iter != servers.end(); iter++)
   {
-    if ((*iter)->has_pid() and not (*iter)->kill((*iter)->pid()))
+    if ((*iter)->check()  == false)
     {
-      Error << "Unable to kill:" <<  *(*iter);
+      success= false;
     }
   }
+
+  return success;
+}
+
+bool server_startup_st::shutdown()
+{
+  bool success= true;
+  for (std::vector<Server *>::iterator iter= servers.begin(); iter != servers.end(); iter++)
+  {
+    if ((*iter)->has_pid() and (*iter)->kill() == false)
+    {
+      Error << "Unable to kill:" <<  *(*iter);
+      success= false;
+    }
+  }
+
+  return success;
 }
 
 void server_startup_st::restart()
@@ -126,13 +160,14 @@ server_startup_st::server_startup_st() :
   _magic(MAGIC_MEMORY),
   _socket(false),
   _sasl(false),
-  _count(5),
-  udp(0)
+  _count(0),
+  udp(0),
+  _servers_to_run(5)
 { }
 
 server_startup_st::~server_startup_st()
 {
-  shutdown_and_remove();
+  clear();
 }
 
 bool server_startup_st::validate()
@@ -140,29 +175,11 @@ bool server_startup_st::validate()
   return _magic == MAGIC_MEMORY;
 }
 
-
-bool server_startup_st::is_debug() const
+bool server_startup(server_startup_st& construct, const std::string& server_type, in_port_t try_port, int argc, const char *argv[], const bool opt_startup_message)
 {
-  return bool(getenv("LIBTEST_MANUAL_GDB"));
-}
-
-bool server_startup_st::is_valgrind() const
-{
-  return bool(getenv("LIBTEST_MANUAL_VALGRIND"));
-}
-
-bool server_startup_st::is_helgrind() const
-{
-  return bool(getenv("LIBTEST_MANUAL_HELGRIND"));
-}
-
-
-bool server_startup(server_startup_st& construct, const std::string& server_type, in_port_t try_port, int argc, const char *argv[])
-{
-  Outn();
   if (try_port <= 0)
   {
-    libtest::fatal(LIBYATL_DEFAULT_PARAM, "was passed the invalid port number %d", int(try_port));
+    throw libtest::fatal(LIBYATL_DEFAULT_PARAM, "was passed the invalid port number %d", int(try_port));
   }
 
   libtest::Server *server= NULL;
@@ -178,6 +195,16 @@ bool server_startup(server_startup_st& construct, const std::string& server_type
       }
     }
   }
+  else if (server_type.compare("drizzled") == 0)
+  {
+    if (DRIZZLED_BINARY)
+    {
+      if (HAVE_LIBDRIZZLE)
+      {
+        server= build_drizzled("localhost", try_port);
+      }
+    }
+  }
   else if (server_type.compare("blobslap_worker") == 0)
   {
     if (GEARMAND_BINARY)
@@ -239,7 +266,7 @@ bool server_startup(server_startup_st& construct, const std::string& server_type
 
   server->build(argc, argv);
 
-  if (construct.is_debug())
+  if (false)
   {
     Out << "Pausing for startup, hit return when ready.";
     std::string gdb_command= server->base_command();
@@ -256,13 +283,16 @@ bool server_startup(server_startup_st& construct, const std::string& server_type
   }
   else
   {
-    Out << "STARTING SERVER(pid:" << server->pid() << "): " << server->running();
+    if (opt_startup_message)
+    {
+      Outn();
+      Out << "STARTING SERVER(pid:" << server->pid() << "): " << server->running();
+      Outn();
+    }
   }
 
   construct.push_server(server);
 
-  Outn();
-
   return true;
 }
 
@@ -337,7 +367,7 @@ bool server_startup_st::start_socket_server(const std::string& server_type, cons
 
   server->build(argc, argv);
 
-  if (is_debug())
+  if (false)
   {
     Out << "Pausing for startup, hit return when ready.";
     std::string gdb_command= server->base_command();
@@ -347,7 +377,7 @@ bool server_startup_st::start_socket_server(const std::string& server_type, cons
 #endif
     getchar();
   }
-  else if (not server->start())
+  else if (server->start() == false)
   {
     Error << "Failed to start " << *server;
     delete server;
