@@ -29,7 +29,7 @@ using namespace libtest;
 static test_return_t collection_init(void *object)
 {
   Context *test= (Context *)object;
-  assert(test);
+  test_true(test);
 
   test_truth(test->initialize(0, NULL));
 
