@@ -42,6 +42,7 @@
 #include <libtest/visibility.h>
 
 #include <boost/thread.hpp>
+#include <boost/shared_ptr.hpp>
 
 
 struct worker_handle_st
@@ -53,16 +54,17 @@ public:
   void set_shutdown();
   bool is_shutdown();
   bool shutdown();
+  void kill();
 
   void set_worker_id(gearman_worker_st*);
 
   boost::barrier* sync_point();
 
   void wait();
-
+  bool check();
 
   volatile bool failed_startup;
-  boost::thread* _thread;
+  boost::shared_ptr<boost::thread> _thread;
 
 private:
   bool _shutdown;
@@ -71,6 +73,22 @@ private:
   boost::barrier _sync_point;
 };
 
+struct worker_handles_st
+{
+  worker_handles_st();
+  ~worker_handles_st();
+
+  // Warning, this will not clean up memory
+  void kill_all();
+
+  void reset();
+
+  void push(worker_handle_st *arg);
+
+private:
+  std::vector<worker_handle_st *> _workers;
+};
+
 #pragma once
 
 LIBTEST_API
@@ -79,7 +97,8 @@ LIBTEST_API
 					     const char *function_name,
 					     gearman_function_t &worker_fn,
 					     void *context,
-					     gearman_worker_options_t options);
+					     gearman_worker_options_t options,
+               int timeout= 0);
 
 LIBTEST_API
 bool test_worker_stop(struct worker_handle_st *);
